<?php
/**
 * @file
 * Defines module classes
 *
 * @ingroup xml_field
 * @{
 */

/*
 * Class xmlFieldXMLElement
 *
 * Allows for proper formatting of CDATA
 *
 * The approach taken is this: any node value that is wrapped in CDATA is
 * stashed in the class static variable, keyed by the instance id, and then
 * later retrieved during the asXml() method.
 */
class xmlFieldXMLElement extends simpleXMLElement {

  public static $cdata_storage = array();

  /**
   * This is an array of setting keyed by the instance id of all instances
   * of this class class.  This technique is used to hold instance settings
   * that are not propogated to the xml output and is due to the fact that
   * we cannot add properties to the SimpleXML object.
   *
   * Even though this is public it should not be accessed, instead you must
   * use the methods setSetting() and getSetting().  This is only public
   * because it has to be to make the static work.
   *
   * @var array
   */
  public static $settings = array();

  /**
   * Returns the cdata value by key.
   *
   * This must be used to directly access a node that is wrapped in cdata.  If
   * the passed node is not wrapped in cdata, then the string value will be
   * returned; so this can be thought of as getter for the node value that
   * casts the node value to a string.
   *
   * @code
   *   $xml->addChild('title', '<![CDATA[<h1>My Book</h1>]]>');
   *   $xml->cdata($xml->title);
   * @endcode
   *
   * @param string|XmlFieldXmlElement $key
   *   E.g. [CDATA[0]] or just: 0
   */
  public function cdata($key = NULL) {
    if (!($cdata = $this->getSetting('cdata_storage'))) {
      $cdata = array();
    }

    if (!isset($key)) {
      return $cdata;
    }

    $key = (string) (is_numeric($key) ? "[CDATA[$key]]" : $key);
    return isset($cdata[$key]) ? $cdata[$key] : $key;
  }

  /**
   * Public setting for the AutoCData setting
   *
   * @param bool $state
   */
  public function setAutoCData($state) {
    return $this->setSetting('autoCData', $state);
  }
  
  public function setSetting($settingName, $value) {
    $hash = spl_object_hash($this);
    self::$settings[$hash][$settingName] = $value;

    return $this;
  }

  public function getSetting($settingName) {
    $hash = spl_object_hash($this);
    return isset(self::$settings[$hash][$settingName]) ? self::$settings[$hash][$settingName] : NULL;
  }
  

  /**
   * Add a child element; this corrects CDATA issues
   */
  public function addChild($name, $value = NULL, $namespace = NULL) {
    
    // Automatic CDATA handling if enabled.
    if ($this->getSetting('autoCData')
      && strpos($value, '<![CDATA[') === FALSE
      && preg_match("/[<&>\"']/", $value)) {
      $value = "<![CDATA[$value]]>";
    }

    // Move CDATA values to $cdata_storage and replace with token
    if (strpos($value, '<![CDATA[') === 0) {
      $cdata = $this->getSetting('cdata_storage');
      $id = count($cdata);
      $id = "[CDATA[$id]]";
      $cdata[$id] = $value;
      $this->setSetting('cdata_storage', $cdata);
      $value = $id;
    }
    switch (func_num_args()) {
      case 1:
        return parent::addChild($name);
      case 2:
        return parent::addChild($name, $value);
      case 3:
        return parent::addChild($name, $value, $namespace);
    }
    //$parent = new ReflectionObject($this);
    //$parent_class = $parent->getParentClass();
    //$ref = new ReflectionMethod($parent_class, 'addChild');
    //$ref->invoke(new)

    // This broke in 5.2.x so the above is wonky but works - aklump
    // 02/08/2012 10:41:08
    //return call_user_func_array('parent::addChild', $args);
  }

  /**
   * Adds a time value as a child and properly formats the time.
   *
   * @param string $name
   * @param mixed $value This is sent to DateObject class.
   * @param string $namespace 
   *
   * @see  DateObject
   *
   * This function depends on the date module being enabled.
   *
   * @see  https://www.drupal.org/project/date
   */
  public function addDateChild($name, $value, $namespace = NULL) {
    $value = $value instanceof DateObject ? $value : new DateObject($value);

    return $this->addChild($name, $value->format('r'), $namespace);
  }

  /**
   * Extends to allow chaining of add attribute
   *
   * e.g. $xml->addAttribute('size', 'large')->addAttribute('color', 'blue')
   */
  public function addAttribute($name, $value = NULL, $namespace = NULL) {
    switch (func_num_args()) {
      case 1:
        parent::addAttribute($name);
        break;
      case 2:
        parent::addAttribute($name, $value);
        break;
      case 3:
        parent::addAttribute($name, $value, $namespace);
        break;
    }
    return $this;
  }

  /**
   * Renders to xml, counterpart to the CDATA issue
   */
  public function asXML($filename = NULL) {
    switch (func_num_args()) {
      case 0:
        $string = parent::asXML();
        break;
      case 1:
        $string = parent::asXML($filename);
        break;
    }
    // Restore the cdata
    if ($this->cdata() && preg_match_all('/\[CDATA\[(\d+)\]\]/', $string, $found)) {
      foreach ($found[0] as $id) {
        $string = str_replace($id, $this->cdata($id), $string);
      }
    }

    return $string;
  }
}

